﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("ParameterVectorMoveMaker", "Applies a move.")]
  [StorableClass]
  public class ParameterVectorMoveMaker : InstrumentedOperator, IParameterVectorMoveOperator, IMoveMaker {
    public ILookupParameter<Configuration> ConfigurationParameter {
      get { return (ILookupParameter<Configuration>)Parameters["Configuration"]; }
    }

    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }

    public ILookupParameter<DoubleValue> MoveQualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveQuality"]; }
    }

    [StorableConstructor]
    protected ParameterVectorMoveMaker(bool deserializing) : base(deserializing) { }
    protected ParameterVectorMoveMaker(ParameterVectorMoveMaker original, Cloner cloner) : base(original, cloner) { }
    public ParameterVectorMoveMaker() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
      Parameters.Add(new LookupParameter<SingleObjectiveScript>("Script", "The script that will execute the evaluation function and define the parameter vector."));
      Parameters.Add(new LookupParameter<Configuration>("Configuration", "An item that holds the problem's configuration."));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The quality of the parameter vector."));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "The quality of the move."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterVectorMoveMaker(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var config = ConfigurationParameter.ActualValue;
      var name = config.Parameters.First().Key;

      var scope = ExecutionContext.Scope.Parent;
      while (!scope.Variables.ContainsKey(name)) {
        if (scope.Parent == null) throw new InvalidOperationException("Scope with solution could not be found!");
        scope = scope.Parent;
      }

      foreach (var param in config.Parameters) {
        if (!scope.Variables.ContainsKey(param.Key) || !ExecutionContext.Scope.Variables.ContainsKey(param.Key))
          throw new InvalidOperationException("Parameter " + param.Key + " not found.");
        scope.Variables[param.Key].Value = ExecutionContext.Scope.Variables[param.Key].Value;
      }

      if (MoveQualityParameter.ActualValue == null) throw new InvalidOperationException("Move has not been evaluated!");
      if (QualityParameter.ActualValue == null) QualityParameter.ActualValue = new DoubleValue(MoveQualityParameter.ActualValue.Value);
      else QualityParameter.ActualValue.Value = MoveQualityParameter.ActualValue.Value;
      return base.InstrumentedApply();
    }
  }
}
